@@ -39,9 +39,22 @@
 #include <inttypes.h>
 #include <string.h>
 #include <assert.h>
+
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
+
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+
+#ifdef HAVE_LINUX_VM_SOCKETS_H
+#include <linux/vm_sockets.h>
+#endif
 
 #include <nbdkit-filter.h>
 
@@ -53,12 +66,16 @@ int ip_debug_rules;
 
 struct rule {
   struct rule *next;
-  enum { BAD = 0, ANY, ANYV4, ANYV6, IPV4, IPV6 } type;
+  enum { BAD = 0,
+         ANY, ANYV4, ANYV6, IPV4, IPV6,
+         ANYUNIX, PID, UID, GID,
+         ANYVSOCK, VSOCKCID, VSOCKPORT } type;
   union {
-    struct in_addr ipv4;
+    struct in_addr ipv4;        /* for IPV4, IPV6 */
     struct in6_addr ipv6;
+    int64_t id;                 /* for PID, UID, GID, VSOCKCID, VSOCKPORT */
   } u;
-  unsigned prefixlen;
+  unsigned prefixlen;           /* for IPV4, IPV6 */
 };
 
 static struct rule *allow_rules, *allow_rules_last;
@@ -91,6 +108,29 @@ print_rule (const char *name, const struct rule *rule, const char *suffix)
     nbdkit_debug ("%s=ipv6:[%s]/%u%s", name, u.addr6, rule->prefixlen, suffix);
     break;
 
+  case ANYUNIX:
+    nbdkit_debug ("%s=anyunix%s", name, suffix);
+    break;
+  case PID:
+    nbdkit_debug ("%s=pid:%" PRIi64 "%s", name, rule->u.id, suffix);
+    break;
+  case UID:
+    nbdkit_debug ("%s=uid:%" PRIi64 "%s", name, rule->u.id, suffix);
+    break;
+  case GID:
+    nbdkit_debug ("%s=gid:%" PRIi64 "%s", name, rule->u.id, suffix);
+    break;
+
+  case ANYVSOCK:
+    nbdkit_debug ("%s=anyvsock%s", name, suffix);
+    break;
+  case VSOCKCID:
+    nbdkit_debug ("%s=vsock-cid:%" PRIi64 "%s", name, rule->u.id, suffix);
+    break;
+  case VSOCKPORT:
+    nbdkit_debug ("%s=vsock-port:%" PRIi64 "%s", name, rule->u.id, suffix);
+    break;
+
   case BAD:
     nbdkit_debug ("%s=BAD(!)%s", name, suffix);
     break;
@@ -218,6 +258,69 @@ parse_rule (const char *paramname,
     return 0;
   }
 
+  if (n == 7 && (ascii_strncasecmp (value, "allunix", 7) == 0 ||
+                 ascii_strncasecmp (value, "anyunix", 7) == 0)) {
+    new_rule->type = ANYUNIX;
+    return 0;
+  }
+
+  if (n >= 4 && ascii_strncasecmp (value, "pid:", 4) == 0) {
+    new_rule->type = PID;
+    if (nbdkit_parse_int64_t ("pid:", &value[4], &new_rule->u.id) == -1)
+      return -1;
+    if (new_rule->u.id <= 0) {
+      nbdkit_error ("pid: parameter out of range");
+      return -1;
+    }
+    return 0;
+  }
+  if (n >= 4 && ascii_strncasecmp (value, "uid:", 4) == 0) {
+    new_rule->type = UID;
+    if (nbdkit_parse_int64_t ("uid:", &value[4], &new_rule->u.id) == -1)
+      return -1;
+    if (new_rule->u.id < 0) {
+      nbdkit_error ("uid: parameter out of range");
+      return -1;
+    }
+    return 0;
+  }
+  if (n >= 4 && ascii_strncasecmp (value, "gid:", 4) == 0) {
+    new_rule->type = GID;
+    if (nbdkit_parse_int64_t ("gid:", &value[4], &new_rule->u.id) == -1)
+      return -1;
+    if (new_rule->u.id < 0) {
+      nbdkit_error ("gid: parameter out of range");
+      return -1;
+    }
+    return 0;
+  }
+
+  if (n == 8 && (ascii_strncasecmp (value, "allvsock", 8) == 0 ||
+                 ascii_strncasecmp (value, "anyvsock", 8) == 0)) {
+    new_rule->type = ANYVSOCK;
+    return 0;
+  }
+  if (n >= 10 && ascii_strncasecmp (value, "vsock-cid:", 10) == 0) {
+    new_rule->type = VSOCKCID;
+    if (nbdkit_parse_int64_t ("vsock-cid:", &value[10], &new_rule->u.id) == -1)
+      return -1;
+    if (new_rule->u.id < 0 || new_rule->u.id > UINT32_MAX) {
+      nbdkit_error ("vsock-cid: parameter out of range");
+      return -1;
+    }
+    return 0;
+  }
+  if (n >= 11 && ascii_strncasecmp (value, "vsock-port:", 11) == 0) {
+    new_rule->type = VSOCKPORT;
+    if (nbdkit_parse_int64_t ("vsock-port:", &value[11], &new_rule->u.id) == -1)
+      return -1;
+    if (new_rule->u.id < 0 || new_rule->u.id > UINT32_MAX) {
+      nbdkit_error ("vsock-port: parameter out of range");
+      return -1;
+    }
+    return 0;
+  }
+
   /* Address with prefixlen. */
   if ((p = strchr (value, '/')) != NULL) {
     size_t pllen = &value[n] - &p[1];
@@ -368,6 +471,9 @@ matches_rule (const struct rule *rule,
   const struct sockaddr_in *sin;
   uint32_t cin, rin, mask;
   const struct sockaddr_in6 *sin6;
+#ifdef AF_VSOCK
+  const struct sockaddr_vm *svm;
+#endif
 
   switch (rule->type) {
   case ANY:
@@ -392,6 +498,43 @@ matches_rule (const struct rule *rule,
     sin6 = (struct sockaddr_in6 *) addr;
     return ipv6_equal (sin6->sin6_addr, rule->u.ipv6, rule->prefixlen);
 
+  case ANYUNIX:
+    return family == AF_UNIX;
+
+    /* Note these work even if the underlying nbdkit_peer_* call fails. */
+  case PID:
+    if (family != AF_UNIX) return false;
+    return nbdkit_peer_pid () == rule->u.id;
+
+  case UID:
+    if (family != AF_UNIX) return false;
+    return nbdkit_peer_uid () == rule->u.id;
+
+  case GID:
+    if (family != AF_UNIX) return false;
+    return nbdkit_peer_gid () == rule->u.id;
+
+#ifdef AF_VSOCK
+  case ANYVSOCK:
+    return family == AF_VSOCK;
+
+  case VSOCKCID:
+    if (family != AF_VSOCK) return false;
+    svm = (struct sockaddr_vm *) addr;
+    return svm->svm_cid == rule->u.id;
+
+  case VSOCKPORT:
+    if (family != AF_VSOCK) return false;
+    svm = (struct sockaddr_vm *) addr;
+    return svm->svm_port == rule->u.id;
+
+#else /* !AF_VSOCK */
+  case ANYVSOCK:
+  case VSOCKCID:
+  case VSOCKPORT:
+    return false;
+#endif
+
   case BAD:
   default:
     abort ();
@@ -421,8 +564,14 @@ check_if_allowed (const struct sockaddr *addr)
 {
   int family = ((struct sockaddr_in *)addr)->sin_family;
 
-  /* There's an implicit allow all for non-IP sockets, see the manual. */
-  if (family != AF_INET && family != AF_INET6)
+  /* There's an implicit allow all for non-IP, non-Unix, non-AF_VSOCK
+   * sockets, see the manual.
+   */
+  if (family != AF_INET && family != AF_INET6 && family != AF_UNIX
+#ifdef AF_VSOCK
+      && family != AF_VSOCK
+#endif
+      )
     return true;
 
   if (matches_rules_list ("ip: match source with allow",
@@ -448,7 +597,7 @@ ip_preconnect (nbdkit_next_preconnect *next, void *nxdata, int readonly)
   /* Follow the rules. */
   if (check_if_allowed ((struct sockaddr *) &addr) == false) {
     nbdkit_error ("client not permitted to connect "
-                  "because of IP address restriction");
+                  "because of source address restriction");
     return -1;
   }
 
