@@ -35,33 +35,43 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
+#include <stdbool.h>
 #include <string.h>
 #include <stdarg.h>
 #include <errno.h>
 #include <inttypes.h>
 #include <pthread.h>
-#include <sys/time.h>
 #include <assert.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <sys/types.h>
 
 #include <nbdkit-filter.h>
 
 #include "cleanup.h"
 #include "utils.h"
+#include "windows-compat.h"
 
-static uint64_t connections;
-static char *logfilename;
-static FILE *logfile;
-static int append;
-static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
+#include "log.h"
+
+/* We use the equivalent of printf ("") several times in this file
+ * which worries GCC.  Ignore these.
+ */
+#pragma GCC diagnostic ignored "-Wformat-zero-length"
+
+uint64_t connections;
+const char *logfilename;
+FILE *logfile;
+const char *logscript;
+int append;
+pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
+pid_t saved_pid;
 
 static void
 log_unload (void)
 {
-  if (logfilename)
+  if (logfile)
     fclose (logfile);
-  free (logfilename);
 }
 
 /* Called for each key=value passed on the command line. */
@@ -70,12 +80,7 @@ log_config (nbdkit_next_config *next, void *nxdata,
             const char *key, const char *value)
 {
   if (strcmp (key, "logfile") == 0) {
-    free (logfilename);
-    logfilename = strdup (value);
-    if (!logfilename) {
-      nbdkit_error ("strdup: %m");
-      return -1;
-    }
+    logfilename = value;
     return 0;
   }
   if (strcmp (key, "logappend") == 0) {
@@ -84,19 +89,17 @@ log_config (nbdkit_next_config *next, void *nxdata,
       return -1;
     return 0;
   }
+  if (strcmp (key, "logscript") == 0) {
+    logscript = value;
+    return 0;
+  }
   return next (nxdata, key, value);
 }
 
-static int
-log_config_complete (nbdkit_next_config_complete *next, void *nxdata)
-{
-  if (!logfilename) {
-    nbdkit_error ("missing logfile= parameter for the log filter");
-    return -1;
-  }
-
-  return next (nxdata);
-}
+#define log_config_help \
+  "logfile=<FILE>               The file to place the log in.\n" \
+  "logappend=<BOOL>             True to append to the log (default false).\n" \
+  "logscript=<SCRIPT>           Script to run for logging."
 
 /* Open the logfile. */
 static int
@@ -104,187 +107,101 @@ log_get_ready (nbdkit_next_get_ready *next, void *nxdata, int thread_model)
 {
   int fd;
 
-  /* Using fopen("ae"/"we") would be more convenient, but as Haiku
-   * still lacks that, use this instead. Atomicity is not essential
-   * here since .config completes before threads that might fork, if
-   * we have to later add yet another fallback to fcntl(fileno()) for
-   * systems without O_CLOEXEC.
-   */
-  fd = open (logfilename,
-             O_CLOEXEC | O_WRONLY | O_CREAT | (append ? O_APPEND : O_TRUNC),
-             0666);
-  if (fd < 0) {
-    nbdkit_error ("open: %s: %m", logfilename);
-    return -1;
-  }
-  logfile = fdopen (fd, append ? "a" : "w");
-  if (!logfile) {
-    nbdkit_error ("fdopen: %s: %m", logfilename);
-    close (fd);
-    return -1;
+  if (logfilename) {
+    /* Using fopen("ae"/"we") would be more convenient, but as Haiku
+     * still lacks that, use this instead. Atomicity is not essential
+     * here since .config completes before threads that might fork, if
+     * we have to later add yet another fallback to fcntl(fileno()) for
+     * systems without O_CLOEXEC.
+     */
+    fd = open (logfilename,
+               O_CLOEXEC | O_WRONLY | O_CREAT | (append ? O_APPEND : O_TRUNC),
+               0666);
+    if (fd < 0) {
+      nbdkit_error ("open: %s: %m", logfilename);
+      return -1;
+    }
+    logfile = fdopen (fd, append ? "a" : "w");
+    if (!logfile) {
+      nbdkit_error ("fdopen: %s: %m", logfilename);
+      close (fd);
+      return -1;
+    }
   }
 
+  saved_pid = getpid ();
+
+  print (NULL, "Ready", "thread_model=%d", thread_model);
   return next (nxdata);
 }
 
-#define log_config_help \
-  "logfile=<FILE>    (required) The file to place the log in.\n" \
-  "logappend=<BOOL>  True to append to the log (default false).\n"
-
-struct handle {
-  uint64_t connection;
-  uint64_t id;
-  char *exportname;
-  int tls;
-};
-
-/* Compute the next id number on the current connection. */
-static uint64_t
-get_id (struct handle *h)
+static int
+log_after_fork (nbdkit_next_after_fork *next, void *nxdata)
 {
-  ACQUIRE_LOCK_FOR_CURRENT_SCOPE(&lock);
-  return ++h->id;
-}
+  /* Only issue this message if we actually fork. */
+  if (getpid () != saved_pid)
+    print (NULL, "Fork", "");
 
-/* Output a timestamp and the log message. */
-static void __attribute__ ((format (printf, 4, 5)))
-output (struct handle *h, const char *act, uint64_t id, const char *fmt, ...)
-{
-  va_list args;
-  struct timeval tv;
-  struct tm tm;
-  char timestamp[27] = "Time unknown";
-
-  /* Logging is best effort, so ignore failure to get timestamp */
-  if (!gettimeofday (&tv, NULL))
-    {
-      size_t s;
-
-      gmtime_r (&tv.tv_sec, &tm);
-      s = strftime (timestamp, sizeof timestamp - sizeof ".000000" + 1,
-                    "%F %T", &tm);
-      assert (s);
-      snprintf (timestamp + s, sizeof timestamp - s, ".%06ld",
-                0L + tv.tv_usec);
-    }
-#ifdef HAVE_FLOCKFILE
-  flockfile (logfile);
-#endif
-  if (h)
-    fprintf (logfile, "%s connection=%" PRIu64 " %s ", timestamp,
-             h->connection, act);
-  else
-    fprintf (logfile, "%s %s ", timestamp, act);
-  if (id)
-    fprintf (logfile, "id=%" PRIu64 " ", id);
-  va_start (args, fmt);
-  vfprintf (logfile, fmt, args);
-  va_end (args);
-  fputc ('\n', logfile);
-  fflush (logfile);
-#ifdef HAVE_FUNLOCKFILE
-  funlockfile (logfile);
-#endif
-}
-
-/* Shared code for a nicer log of return value */
-static void
-output_return (struct handle *h, const char *act, uint64_t id, int r, int *err)
-{
-  const char *s = "Other=>EINVAL";
-
-  /* Only decode what protocol.c:nbd_errno() recognizes */
-  if (r == -1) {
-    switch (*err) {
-    case EROFS:
-      s = "EROFS=>EPERM";
-      break;
-    case EPERM:
-      s = "EPERM";
-      break;
-    case EIO:
-      s = "EIO";
-      break;
-    case ENOMEM:
-      s = "ENOMEM";
-      break;
-#ifdef EDQUOT
-    case EDQUOT:
-      s = "EDQUOT=>ENOSPC";
-      break;
-#endif
-    case EFBIG:
-      s = "EFBIG=>ENOSPC";
-      break;
-    case ENOSPC:
-      s = "ENOSPC";
-      break;
-#ifdef ESHUTDOWN
-    case ESHUTDOWN:
-      s = "ESHUTDOWN";
-      break;
-#endif
-    case ENOTSUP:
-#if ENOTSUP != EOPNOTSUPP
-    case EOPNOTSUPP:
-#endif
-      s = "ENOTSUP";
-      break;
-    case EOVERFLOW:
-      s = "EOVERFLOW";
-      break;
-    case EINVAL:
-      s = "EINVAL";
-      break;
-    }
-  }
-  else {
-    s = "Success";
-  }
-  output (h, act, id, "return=%d (%s)", r, s);
+  return next (nxdata);
 }
 
 /* List exports. */
 static int
 log_list_exports (nbdkit_next_list_exports *next, void *nxdata,
-                  int readonly, int default_only,
+                  int readonly, int is_tls,
                   struct nbdkit_exports *exports)
 {
-  static uint64_t id;
+  static log_id_t id;
   int r;
   int err;
 
-  output (NULL, "ListExports", ++id, "readonly=%d default_only=%d ...",
-          readonly, default_only);
-  r = next (nxdata, readonly, default_only, exports);
+  enter (NULL, ++id, "ListExports", "readonly=%d tls=%d", readonly, is_tls);
+  r = next (nxdata, readonly, exports);
   if (r == -1) {
     err = errno;
-    output_return (NULL, "...ListExports", id, r, &err);
+    leave_simple (NULL, id, "ListExports", r, &err);
   }
   else {
     FILE *fp;
-    CLEANUP_FREE char *exports_str = NULL;
+    CLEANUP_FREE char *str = NULL;
     size_t i, n, len = 0;
 
-    fp = open_memstream (&exports_str, &len);
+    fp = open_memstream (&str, &len);
     if (fp != NULL) {
+      fprintf (fp, "exports=(");
       n = nbdkit_exports_count (exports);
       for (i = 0; i < n; ++i) {
         struct nbdkit_export e = nbdkit_get_export (exports, i);
         if (i > 0)
-          fprintf (fp, ", ");
+          fprintf (fp, " ");
         shell_quote (e.name, fp);
       }
-
+      fprintf (fp, ") return=0");
       fclose (fp);
+      leave (NULL, id, "ListExports", "%s", str);
     }
-
-    output (NULL, "...ListExports", id, "exports=[%s] return=0",
-            exports_str ? exports_str : "(null)");
+    else
+      leave (NULL, id, "ListExports", "");
   }
   return r;
 }
 
+static int
+log_preconnect (nbdkit_next_preconnect *next, nbdkit_backend *nxdata,
+                int readonly)
+{
+  static log_id_t id;
+  int r;
+  int err;
+
+  enter (NULL, ++id, "Preconnect", "readonly=%d", readonly);
+  r = next (nxdata, readonly);
+  if (r == -1)
+    err = errno;
+  leave_simple (NULL, id, "Preconnect", r, &err);
+  return r;
+}
+
 /* Open a connection. */
 static void *
 log_open (nbdkit_next_open *next, void *nxdata,
@@ -305,9 +222,8 @@ log_open (nbdkit_next_open *next, void *nxdata,
    * it in log_prepare.  We must take a copy because this string has a
    * short lifetime.
    */
-  h->exportname = strdup (exportname);
+  h->exportname = nbdkit_strdup_intern (exportname);
   if (h->exportname == NULL) {
-    nbdkit_error ("strdup: %m");
     free (h);
     return NULL;
   }
@@ -322,16 +238,16 @@ log_open (nbdkit_next_open *next, void *nxdata,
 static void
 log_close (void *handle)
 {
-  struct handle *h = handle;
-
-  free (h->exportname);
-  free (h);
+  free (handle);
 }
 
 static int
 log_prepare (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle,
              int readonly)
 {
+  FILE *fp;
+  CLEANUP_FREE char *str = NULL;
+  size_t len = 0;
   struct handle *h = handle;
   const char *exportname = h->exportname;
   int64_t size = next_ops->get_size (nxdata);
@@ -349,9 +265,21 @@ log_prepare (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle,
       e < 0 || c < 0 || Z < 0)
     return -1;
 
-  output (h, "Connect", 0, "export='%s' tls=%d size=0x%" PRIx64 " write=%d "
-          "flush=%d rotational=%d trim=%d zero=%d fua=%d extents=%d cache=%d "
-          "fast_zero=%d", exportname, h->tls, size, w, f, r, t, z, F, e, c, Z);
+  fp = open_memstream (&str, &len);
+  if (fp) {
+    fprintf (fp, "export=");
+    shell_quote (exportname, fp);
+    fprintf (fp,
+             " tls=%d size=0x%" PRIx64 " write=%d "
+             "flush=%d rotational=%d trim=%d zero=%d fua=%d extents=%d "
+             "cache=%d fast_zero=%d",
+             h->tls, size, w, f, r, t, z, F, e, c, Z);
+    fclose (fp);
+    print (h, "Connect", "%s", str);
+  }
+  else
+    print (h, "Connect", "");
+
   return 0;
 }
 
@@ -360,7 +288,7 @@ log_finalize (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle)
 {
   struct handle *h = handle;
 
-  output (h, "Disconnect", 0, "transactions=%" PRId64, h->id);
+  print (h, "Disconnect", "transactions=%" PRId64, h->id);
   return 0;
 }
 
@@ -371,15 +299,12 @@ log_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
            uint32_t flags, int *err)
 {
   struct handle *h = handle;
-  uint64_t id = get_id (h);
   int r;
 
+  LOG (h, "Read", r, err, "offset=0x%" PRIx64 " count=0x%x", offs, count);
+
   assert (!flags);
-  output (h, "Read", id, "offset=0x%" PRIx64 " count=0x%x ...",
-          offs, count);
-  r = next_ops->pread (nxdata, buf, count, offs, flags, err);
-  output_return (h, "...Read", id, r, err);
-  return r;
+  return r = next_ops->pread (nxdata, buf, count, offs, flags, err);
 }
 
 /* Write data. */
@@ -389,15 +314,14 @@ log_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
             uint32_t flags, int *err)
 {
   struct handle *h = handle;
-  uint64_t id = get_id (h);
   int r;
 
+  LOG (h, "Write", r, err,
+       "offset=0x%" PRIx64 " count=0x%x fua=%d",
+       offs, count, !!(flags & NBDKIT_FLAG_FUA));
+
   assert (!(flags & ~NBDKIT_FLAG_FUA));
-  output (h, "Write", id, "offset=0x%" PRIx64 " count=0x%x fua=%d ...",
-          offs, count, !!(flags & NBDKIT_FLAG_FUA));
-  r = next_ops->pwrite (nxdata, buf, count, offs, flags, err);
-  output_return (h, "...Write", id, r, err);
-  return r;
+  return r = next_ops->pwrite (nxdata, buf, count, offs, flags, err);
 }
 
 /* Flush. */
@@ -406,14 +330,12 @@ log_flush (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle,
            uint32_t flags, int *err)
 {
   struct handle *h = handle;
-  uint64_t id = get_id (h);
   int r;
 
+  LOG (h, "Flush", r, err, "");
+
   assert (!flags);
-  output (h, "Flush", id, "...");
-  r = next_ops->flush (nxdata, flags, err);
-  output_return (h, "...Flush", id, r, err);
-  return r;
+  return r = next_ops->flush (nxdata, flags, err);
 }
 
 /* Trim data. */
@@ -423,15 +345,14 @@ log_trim (struct nbdkit_next_ops *next_ops, void *nxdata,
           int *err)
 {
   struct handle *h = handle;
-  uint64_t id = get_id (h);
   int r;
 
+  LOG (h, "Trim", r, err,
+       "offset=0x%" PRIx64 " count=0x%x fua=%d",
+       offs, count, !!(flags & NBDKIT_FLAG_FUA));
+
   assert (!(flags & ~NBDKIT_FLAG_FUA));
-  output (h, "Trim", id, "offset=0x%" PRIx64 " count=0x%x fua=%d ...",
-          offs, count, !!(flags & NBDKIT_FLAG_FUA));
-  r = next_ops->trim (nxdata, count, offs, flags, err);
-  output_return (h, "...Trim", id, r, err);
-  return r;
+  return r = next_ops->trim (nxdata, count, offs, flags, err);
 }
 
 /* Zero data. */
@@ -441,19 +362,17 @@ log_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
           int *err)
 {
   struct handle *h = handle;
-  uint64_t id = get_id (h);
   int r;
 
+  LOG (h, "Zero", r, err,
+       "offset=0x%" PRIx64 " count=0x%x trim=%d fua=%d fast=%d",
+       offs, count, !!(flags & NBDKIT_FLAG_MAY_TRIM),
+       !!(flags & NBDKIT_FLAG_FUA),
+       !!(flags & NBDKIT_FLAG_FAST_ZERO));
+
   assert (!(flags & ~(NBDKIT_FLAG_FUA | NBDKIT_FLAG_MAY_TRIM |
                       NBDKIT_FLAG_FAST_ZERO)));
-  output (h, "Zero", id,
-          "offset=0x%" PRIx64 " count=0x%x trim=%d fua=%d fast=%d...",
-          offs, count, !!(flags & NBDKIT_FLAG_MAY_TRIM),
-          !!(flags & NBDKIT_FLAG_FUA),
-          !!(flags & NBDKIT_FLAG_FAST_ZERO));
-  r = next_ops->zero (nxdata, count, offs, flags, err);
-  output_return (h, "...Zero", id, r, err);
-  return r;
+  return r = next_ops->zero (nxdata, count, offs, flags, err);
 }
 
 /* Extents. */
@@ -463,40 +382,48 @@ log_extents (struct nbdkit_next_ops *next_ops, void *nxdata,
              struct nbdkit_extents *extents, int *err)
 {
   struct handle *h = handle;
-  uint64_t id = get_id (h);
+  log_id_t id = get_id (h);
   int r;
 
   assert (!(flags & ~(NBDKIT_FLAG_REQ_ONE)));
-  output (h, "Extents", id,
-          "offset=0x%" PRIx64 " count=0x%x req_one=%d ...",
-          offs, count, !!(flags & NBDKIT_FLAG_REQ_ONE));
+  enter (h, id, "Extents",
+         "offset=0x%" PRIx64 " count=0x%x req_one=%d",
+         offs, count, !!(flags & NBDKIT_FLAG_REQ_ONE));
   r = next_ops->extents (nxdata, count, offs, flags, extents, err);
   if (r == -1)
-    output_return (h, "...Extents", id, r, err);
+    leave_simple (h, id, "Extents", r, err);
   else {
     FILE *fp;
-    CLEANUP_FREE char *extents_str = NULL;
+    CLEANUP_FREE char *str = NULL;
     size_t i, n, len = 0;
 
-    fp = open_memstream (&extents_str, &len);
+    fp = open_memstream (&str, &len);
     if (fp != NULL) {
+      fprintf (fp, "extents=(");
       n = nbdkit_extents_count (extents);
       for (i = 0; i < n; ++i) {
+        bool comma = false;
         struct nbdkit_extent e = nbdkit_get_extent (extents, i);
         if (i > 0)
-          fprintf (fp, ", ");
-        fprintf (fp, "{ offset=0x%" PRIx64 ", length=0x%" PRIx64 ", "
-                 "hole=%d, zero=%d }",
-                 e.offset, e.length,
-                 !!(e.type & NBDKIT_EXTENT_HOLE),
-                 !!(e.type & NBDKIT_EXTENT_ZERO));
+          fprintf (fp, " ");
+        fprintf (fp, "0x%" PRIx64 " 0x%" PRIx64, e.offset, e.length);
+        fprintf (fp, " \"");
+        if ((e.type & NBDKIT_EXTENT_HOLE) != 0) {
+          fprintf (fp, "hole");
+          comma = true;
+        }
+        if ((e.type & NBDKIT_EXTENT_ZERO) != 0) {
+          if (comma) fprintf (fp, ",");
+          fprintf (fp, "zero");
+        }
+        fprintf (fp, "\"");
       }
-
+      fprintf (fp, ") return=0");
       fclose (fp);
+      leave (h, id, "Extents", "%s", str);
     }
-
-    output (h, "...Extents", id, "extents=[%s] return=0",
-            extents_str ? extents_str : "(null)");
+    else
+      leave (h, id, "Extents", "");
   }
   return r;
 }
@@ -508,26 +435,24 @@ log_cache (struct nbdkit_next_ops *next_ops, void *nxdata,
            int *err)
 {
   struct handle *h = handle;
-  uint64_t id = get_id (h);
   int r;
 
+  LOG (h, "Cache", r, err, "offset=0x%" PRIx64 " count=0x%x", offs, count);
+
   assert (!flags);
-  output (h, "Cache", id, "offset=0x%" PRIx64 " count=0x%x ...",
-          offs, count);
-  r = next_ops->cache (nxdata, count, offs, flags, err);
-  output_return (h, "...Cache", id, r, err);
-  return r;
+  return r = next_ops->cache (nxdata, count, offs, flags, err);
 }
 
 static struct nbdkit_filter filter = {
   .name              = "log",
   .longname          = "nbdkit log filter",
   .config            = log_config,
-  .config_complete   = log_config_complete,
   .config_help       = log_config_help,
   .unload            = log_unload,
   .get_ready         = log_get_ready,
+  .after_fork        = log_after_fork,
   .list_exports      = log_list_exports,
+  .preconnect        = log_preconnect,
   .open              = log_open,
   .close             = log_close,
   .prepare           = log_prepare,
