@@ -36,12 +36,12 @@
 #include <stdlib.h>
 #include <stdbool.h>
 #include <string.h>
-#include <ctype.h>
 #include <inttypes.h>
 #include <assert.h>
 
 #include <nbdkit-plugin.h>
 
+#include "ascii-ctype.h"
 #include "byte-swapping.h"
 
 #include "efi-crc32.h"
@@ -88,7 +88,7 @@ create_gpt_partition_header (const void *pt, bool is_primary,
   uint64_t nr_lbas;
   struct gpt_header *header = (struct gpt_header *) out;
 
-  nr_lbas = virtual_size (&regions) / SECTOR_SIZE;
+  nr_lbas = virtual_size (&the_regions) / SECTOR_SIZE;
 
   memset (header, 0, sizeof *header);
   memcpy (header->signature, GPT_SIGNATURE, sizeof (header->signature));
@@ -122,14 +122,14 @@ create_gpt_partition_table (unsigned char *out)
 {
   size_t i, j;
 
-  for (j = 0; j < nr_regions (&regions); ++j) {
-    const struct region *region = get_region (&regions, j);
+  for (j = 0; j < nr_regions (&the_regions); ++j) {
+    const struct region *region = &the_regions.ptr[j];
 
     if (region->type == region_file) {
       i = region->u.i;
       assert (i < GPT_PTA_SIZE);
       create_gpt_partition_table_entry (region, i == 0,
-                                        files[i].type_guid,
+                                        the_files.ptr[i].type_guid,
                                         out);
       out += GPT_PT_ENTRY_SIZE;
     }
@@ -150,7 +150,7 @@ create_gpt_partition_table_entry (const struct region *region,
 
   memcpy (entry->partition_type_guid, partition_type_guid, 16);
 
-  memcpy (entry->unique_guid, files[region->u.i].guid, 16);
+  memcpy (entry->unique_guid, the_files.ptr[region->u.i].guid, 16);
 
   entry->first_lba = htole64 (region->start / SECTOR_SIZE);
   entry->last_lba = htole64 (region->end / SECTOR_SIZE);
@@ -162,7 +162,7 @@ create_gpt_partition_table_entry (const struct region *region,
    * Is this a security risk?  It reveals something about paths on the
    * server to clients. XXX
    */
-  filename = files[region->u.i].filename;
+  filename = the_files.ptr[region->u.i].filename;
   len = strlen (filename);
   if (len < 36) {
     for (i = 0; i < len; ++i)
@@ -188,7 +188,7 @@ create_gpt_protective_mbr (unsigned char *out)
    * expressible with MBR.
    */
   region.start = 512;
-  end = virtual_size (&regions) - 1;
+  end = virtual_size (&the_regions) - 1;
   if (end > UINT32_MAX * SECTOR_SIZE)
     end = UINT32_MAX * SECTOR_SIZE;
   region.end = end;
@@ -244,19 +244,19 @@ parse_guid (const char *str, char *out)
     return -1;
 
   for (i = 0; i < 8; ++i)
-    if (!isxdigit (str[i]))
+    if (!ascii_isxdigit (str[i]))
       return -1;
   for (i = 9; i < 13; ++i)
-    if (!isxdigit (str[i]))
+    if (!ascii_isxdigit (str[i]))
       return -1;
   for (i = 14; i < 18; ++i)
-    if (!isxdigit (str[i]))
+    if (!ascii_isxdigit (str[i]))
       return -1;
   for (i = 19; i < 23; ++i)
-    if (!isxdigit (str[i]))
+    if (!ascii_isxdigit (str[i]))
       return -1;
   for (i = 24; i < 36; ++i)
-    if (!isxdigit (str[i]))
+    if (!ascii_isxdigit (str[i]))
       return -1;
 
   /* The first, second and third blocks are parsed as little endian,
