@@ -62,12 +62,12 @@ create_mbr_layout (void)
   primary[0x1fe] = 0x55;
   primary[0x1ff] = 0xaa;
 
-  if (nr_files <= 4) {
+  if (the_files.size <= 4) {
     /* Basic MBR with no extended partition. */
-    for (i = 0; i < nr_files; ++i) {
+    for (i = 0; i < the_files.size; ++i) {
       const struct region *region = find_file_region (i, &j);
 
-      create_mbr_partition_table_entry (region, i == 0, files[i].mbr_id,
+      create_mbr_partition_table_entry (region, i == 0, the_files.ptr[i].mbr_id,
                                         &primary[0x1be + 16*i]);
     }
   }
@@ -80,7 +80,7 @@ create_mbr_layout (void)
      */
     for (i = 0; i < 3; ++i) {
       rptr = find_file_region (i, &j);
-      create_mbr_partition_table_entry (rptr, i == 0, files[i].mbr_id,
+      create_mbr_partition_table_entry (rptr, i == 0, the_files.ptr[i].mbr_id,
                                         &primary[0x1be + 16*i]);
     }
 
@@ -90,14 +90,14 @@ create_mbr_layout (void)
      */
     eptr0 = find_ebr_region (3, &j);
     region.start = eptr0->start;
-    region.end = virtual_size (&regions) - 1; /* to end of disk */
+    region.end = virtual_size (&the_regions) - 1; /* to end of disk */
     region.len = region.end - region.start + 1;
     create_mbr_partition_table_entry (&region, false, 0xf, &primary[0x1ee]);
 
     /* The remaining files are mapped to logical partitions living in
      * the fourth extended partition.
      */
-    for (i = 3; i < nr_files; ++i) {
+    for (i = 3; i < the_files.size; ++i) {
       if (i == 3)
         eptr = eptr0;
       else
@@ -114,10 +114,10 @@ create_mbr_layout (void)
        */
       region.start = rptr->start - eptr->start;
       region.len = rptr->len;
-      create_mbr_partition_table_entry (&region, false, files[i].mbr_id,
+      create_mbr_partition_table_entry (&region, false, the_files.ptr[i].mbr_id,
                                         &ebr[i-3][0x1be]);
 
-      if (i < nr_files-1) {
+      if (i < the_files.size-1) {
         size_t j2 = j;
         const struct region *enext = find_ebr_region (i+1, &j2);
         const struct region *rnext = find_file_region (i+1, &j2);
@@ -144,8 +144,8 @@ find_file_region (size_t i, size_t *j)
 {
   const struct region *region;
 
-  for (; *j < nr_regions (&regions); ++(*j)) {
-    region = get_region (&regions, *j);
+  for (; *j < nr_regions (&the_regions); ++(*j)) {
+    region = &the_regions.ptr[*j];
     if (region->type == region_file && region->u.i == i)
       return region;
   }
@@ -162,8 +162,8 @@ find_ebr_region (size_t i, size_t *j)
 
   assert (i >= 3);
 
-  for (; *j < nr_regions (&regions); ++(*j)) {
-    region = get_region (&regions, *j);
+  for (; *j < nr_regions (&the_regions); ++(*j)) {
+    region = &the_regions.ptr[*j];
     if (region->type == region_data && region->u.data == ebr[i-3])
       return region;
   }
