@@ -284,44 +284,47 @@ sh_config_complete (void)
 #define THREAD_MODEL NBDKIT_THREAD_MODEL_PARALLEL
 
 static struct nbdkit_plugin plugin = {
-  .name              = "sh",
-  .version           = PACKAGE_VERSION,
-  .load              = sh_load,
-  .unload            = sh_unload,
+  .name               = "sh",
+  .version            = PACKAGE_VERSION,
+  .load               = sh_load,
+  .unload             = sh_unload,
 
-  .dump_plugin       = sh_dump_plugin,
+  .dump_plugin        = sh_dump_plugin,
 
-  .config            = sh_config,
-  .config_complete   = sh_config_complete,
-  .config_help       = sh_config_help,
-  .magic_config_key  = "script",
-  .thread_model      = sh_thread_model,
-  .get_ready         = sh_get_ready,
-  .after_fork        = sh_after_fork,
+  .config             = sh_config,
+  .config_complete    = sh_config_complete,
+  .config_help        = sh_config_help,
+  .magic_config_key   = "script",
+  .thread_model       = sh_thread_model,
+  .get_ready          = sh_get_ready,
+  .after_fork         = sh_after_fork,
 
-  .preconnect        = sh_preconnect,
-  .open              = sh_open,
-  .close             = sh_close,
+  .preconnect         = sh_preconnect,
+  .list_exports       = sh_list_exports,
+  .default_export     = sh_default_export,
+  .open               = sh_open,
+  .close              = sh_close,
 
-  .get_size          = sh_get_size,
-  .can_write         = sh_can_write,
-  .can_flush         = sh_can_flush,
-  .is_rotational     = sh_is_rotational,
-  .can_trim          = sh_can_trim,
-  .can_zero          = sh_can_zero,
-  .can_extents       = sh_can_extents,
-  .can_fua           = sh_can_fua,
-  .can_multi_conn    = sh_can_multi_conn,
-  .can_cache         = sh_can_cache,
-  .can_fast_zero     = sh_can_fast_zero,
+  .export_description = sh_export_description,
+  .get_size           = sh_get_size,
+  .can_write          = sh_can_write,
+  .can_flush          = sh_can_flush,
+  .is_rotational      = sh_is_rotational,
+  .can_trim           = sh_can_trim,
+  .can_zero           = sh_can_zero,
+  .can_extents        = sh_can_extents,
+  .can_fua            = sh_can_fua,
+  .can_multi_conn     = sh_can_multi_conn,
+  .can_cache          = sh_can_cache,
+  .can_fast_zero      = sh_can_fast_zero,
 
-  .pread             = sh_pread,
-  .pwrite            = sh_pwrite,
-  .flush             = sh_flush,
-  .trim              = sh_trim,
-  .zero              = sh_zero,
-  .extents           = sh_extents,
-  .cache             = sh_cache,
+  .pread              = sh_pread,
+  .pwrite             = sh_pwrite,
+  .flush              = sh_flush,
+  .trim               = sh_trim,
+  .zero               = sh_zero,
+  .extents            = sh_extents,
+  .cache              = sh_cache,
 
   .errno_is_preserved = 1,
 };
