@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2019 Red Hat Inc.
+ * Copyright (C) 2013-2020 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -40,14 +40,10 @@
  */
 
 /* Required stubs, present in all versions of VDDK that we support.  I
- * have checked that all these exist in at least VDDK 5.1.1 (2011)
+ * have checked that all these exist in at least VDDK 5.5.5 (2015)
  * which is the earliest version of VDDK that we support.
  */
 
-/* We treat these two stubs slightly specially for the benefit of
- * tests/dummy-vddk.c.
- */
-#ifndef NO_INITEX_STUB
 STUB (VixDiskLib_InitEx,
       VixError,
       (uint32_t major, uint32_t minor,
@@ -56,13 +52,9 @@ STUB (VixDiskLib_InitEx,
        VixDiskLibGenericLogFunc *panic_function,
        const char *lib_dir,
        const char *config_file));
-#endif
-#ifndef NO_EXIT_STUB
 STUB (VixDiskLib_Exit,
       void,
       (void));
-#endif
-
 STUB (VixDiskLib_GetErrorText,
       char *,
       (VixError err, const char *unused));
