@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2019 Red Hat Inc.
+ * Copyright (C) 2013-2020 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -51,6 +51,7 @@ enum {
   RUN_OPTION,
   SELINUX_LABEL_OPTION,
   SHORT_OPTIONS_OPTION,
+  SWAP_OPTION,
   TLS_OPTION,
   TLS_CERTIFICATES_OPTION,
   TLS_PSK_OPTION,
@@ -92,6 +93,7 @@ static const struct option long_options[] = {
   { "short-options",    no_argument,       NULL, SHORT_OPTIONS_OPTION },
   { "single",           no_argument,       NULL, 's' },
   { "stdin",            no_argument,       NULL, 's' },
+  { "swap",             no_argument,       NULL, SWAP_OPTION },
   { "threads",          required_argument, NULL, 't' },
   { "tls",              required_argument, NULL, TLS_OPTION },
   { "tls-certificates", required_argument, NULL, TLS_CERTIFICATES_OPTION },
