@@ -45,18 +45,43 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <limits.h>
-#include <termios.h>
 #include <errno.h>
-#include <poll.h>
 #include <signal.h>
+#include <sys/types.h>
+
+#ifdef HAVE_TERMIOS_H
+#include <termios.h>
+#endif
+
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+
+#ifdef HAVE_SYS_UCRED_H
+#include <sys/ucred.h>
+#endif
+
+#ifdef HAVE_SYS_UN_H
+#include <sys/un.h>
+#endif
+
+#ifdef WIN32
+/* For nanosleep on Windows. */
+#include <pthread_time.h>
+#endif
 
 #include "ascii-ctype.h"
 #include "ascii-string.h"
-#include "get-current-dir-name.h"
+#include "get_current_dir_name.h"
+#include "getline.h"
+#include "poll.h"
+#include "realpath.h"
+#include "strndup.h"
 
 #include "internal.h"
 
+#ifndef WIN32
+
 char *
 nbdkit_absolute_path (const char *path)
 {
@@ -83,7 +108,7 @@ nbdkit_absolute_path (const char *path)
     return NULL;
   }
 
-  if (asprintf (&ret, "%s/%s", pwd, path) == -1) {
+  if (asprintf (&ret, "%s" DIR_SEPARATOR_STR "%s", pwd, path) == -1) {
     nbdkit_error ("asprintf: %m");
     return NULL;
   }
@@ -91,6 +116,21 @@ nbdkit_absolute_path (const char *path)
   return ret;
 }
 
+#else /* WIN32 */
+
+/* On Windows realpath() is replaced by GetFullPathName which doesn't
+ * bother to check if the final path exists.  Therefore we can simply
+ * replace nbdkit_absolute_path with nbdkit_realpath and everything
+ * should work the same.
+ */
+char *
+nbdkit_absolute_path (const char *path)
+{
+  return nbdkit_realpath (path);
+}
+
+#endif /* WIN32 */
+
 char *
 nbdkit_realpath (const char *path)
 {
@@ -429,6 +469,7 @@ nbdkit_read_password (const char *value, char **password)
 
   /* Read from numbered file descriptor. */
   else if (value[0] == '-') {
+#ifndef WIN32
     int fd;
 
     if (nbdkit_parse_int ("password file descriptor", &value[1], &fd) == -1)
@@ -439,6 +480,15 @@ nbdkit_read_password (const char *value, char **password)
     }
     if (read_password_from_fd (&value[1], fd, password) == -1)
       return -1;
+
+#else /* WIN32 */
+    /* As far as I know this will never be possible on Windows, so
+     * it's a simple error.
+     */
+    nbdkit_error ("not possible to read passwords from file descriptors "
+                  "under Windows");
+    return -1;
+#endif /* WIN32 */
   }
 
   /* Read password from a file. */
@@ -466,6 +516,8 @@ nbdkit_read_password (const char *value, char **password)
   return 0;
 }
 
+#ifndef WIN32
+
 typedef struct termios echo_mode;
 
 static void
@@ -485,6 +537,37 @@ echo_restore (const echo_mode *old_mode)
   tcsetattr (STDIN_FILENO, TCSAFLUSH, old_mode);
 }
 
+#else /* WIN32 */
+
+/* Windows implementation of tty echo off based on this:
+ * https://stackoverflow.com/a/1455007
+ */
+typedef DWORD echo_mode;
+
+static void
+echo_off (echo_mode *old_mode)
+{
+  HANDLE h_stdin;
+  DWORD mode;
+
+  h_stdin = GetStdHandle (STD_INPUT_HANDLE);
+  GetConsoleMode (h_stdin, old_mode);
+  mode = *old_mode;
+  mode &= ~ENABLE_ECHO_INPUT;
+  SetConsoleMode (h_stdin, mode);
+}
+
+static void
+echo_restore (const echo_mode *old_mode)
+{
+  HANDLE h_stdin;
+
+  h_stdin = GetStdHandle (STD_INPUT_HANDLE);
+  SetConsoleMode (h_stdin, *old_mode);
+}
+
+#endif /* WIN32 */
+
 static int
 read_password_interactive (char **password)
 {
@@ -726,3 +809,250 @@ nbdkit_peer_name (struct sockaddr *addr, socklen_t *addrlen)
 
   return 0;
 }
+
+#ifdef SO_PEERCRED
+
+static int
+get_peercred (int s, int64_t *pid, int64_t *uid, int64_t *gid)
+{
+  struct ucred ucred;
+  socklen_t n = sizeof ucred;
+
+  if (getsockopt (s, SOL_SOCKET, SO_PEERCRED, &ucred, &n) == -1) {
+    nbdkit_error ("getsockopt: SO_PEERCRED: %m");
+    return -1;
+  }
+
+  if (pid && ucred.pid >= 1) {
+    if (ucred.pid <= INT64_MAX)
+      *pid = ucred.pid;
+    else
+      nbdkit_error ("pid out of range: cannot be mapped to int64_t");
+  }
+  if (uid && ucred.uid >= 0) {
+    if (ucred.uid <= INT64_MAX)
+      *uid = ucred.uid;
+    else
+      nbdkit_error ("uid out of range: cannot be mapped to int64_t");
+  }
+  if (gid && ucred.gid >= 0) {
+    if (ucred.gid <= INT64_MAX)
+      *gid = ucred.gid;
+    else
+      nbdkit_error ("gid out of range: cannot be mapped to int64_t");
+  }
+
+  return 0;
+}
+
+#endif /* SO_PEERCRED */
+
+#ifdef LOCAL_PEERCRED
+
+/* FreeBSD supports LOCAL_PEERCRED and struct xucred. */
+static int
+get_peercred (int s, int64_t *pid, int64_t *uid, int64_t *gid)
+{
+  struct xucred xucred;
+  socklen_t n = sizeof xucred;
+
+  if (getsockopt (s, 0, LOCAL_PEERCRED, &xucred, &n) == -1) {
+    nbdkit_error ("getsockopt: LOCAL_PEERCRED: %m");
+    return -1;
+  }
+
+  if (xucred.cr_version != XUCRED_VERSION) {
+    nbdkit_error ("getsockopt: LOCAL_PEERCRED: "
+                  "struct xucred version (%u) "
+                  "did not match expected version (%u)",
+                  xucred.cr_version, XUCRED_VERSION);
+    return -1;
+  }
+
+  if (n != sizeof xucred) {
+    nbdkit_error ("getsockopt: LOCAL_PEERCRED: did not return full struct");
+    return -1;
+  }
+
+  if (pid)
+    nbdkit_error ("nbdkit_peer_pid is not supported on this platform");
+  if (uid && xucred.cr_uid >= 0) {
+    if (xucred.cr_uid <= INT64_MAX)
+      *uid = xucred.cr_uid;
+    else
+      nbdkit_error ("uid out of range: cannot be mapped to int64_t");
+  }
+  if (gid && xucred.cr_ngroups > 0) {
+    if (xucred.cr_gid <= INT64_MAX)
+      *gid = xucred.cr_gid;
+    else
+      nbdkit_error ("gid out of range: cannot be mapped to int64_t");
+  }
+
+  return 0;
+}
+
+#endif /* LOCAL_PEERCRED */
+
+#if !defined(SO_PEERCRED) && !defined(LOCAL_PEERCRED)
+
+static int
+get_peercred (int s, int64_t *pid, int64_t *uid, int64_t *gid)
+{
+  nbdkit_error ("nbdkit_peer_pid, nbdkit_peer_uid and nbdkit_peer_gid "
+                "are not supported on this platform");
+  return -1;
+}
+
+#endif
+
+static int
+get_peercred_common (int64_t *pid, int64_t *uid, int64_t *gid)
+{
+  struct connection *conn = threadlocal_get_conn ();
+  int s;
+
+  if (pid) *pid = -1;
+  if (uid) *uid = -1;
+  if (gid) *gid = -1;
+
+  if (!conn) {
+    nbdkit_error ("no connection in this thread");
+    return -1;
+  }
+
+  s = conn->sockin;
+  if (s == -1) {
+    nbdkit_error ("socket not open");
+    return -1;
+  }
+
+  return get_peercred (s, pid, uid, gid);
+}
+
+int64_t
+nbdkit_peer_pid ()
+{
+  int64_t pid;
+
+  if (get_peercred_common (&pid, NULL, NULL) == -1)
+    return -1;
+
+  return pid;
+}
+
+int64_t
+nbdkit_peer_uid ()
+{
+  int64_t uid;
+
+  if (get_peercred_common (NULL, &uid, NULL) == -1)
+    return -1;
+
+  return uid;
+}
+
+int64_t
+nbdkit_peer_gid ()
+{
+  int64_t gid;
+
+  if (get_peercred_common (NULL, NULL, &gid) == -1)
+    return -1;
+
+  return gid;
+}
+
+/* Functions for manipulating intern'd strings. */
+
+static string_vector global_interns;
+
+void
+free_interns (void)
+{
+  struct connection *conn = threadlocal_get_conn ();
+  string_vector *list = conn ? &conn->interns : &global_interns;
+
+  string_vector_iter (list, (void *) free);
+  string_vector_reset (list);
+}
+
+static const char *
+add_intern (char *str)
+{
+  struct connection *conn = threadlocal_get_conn ();
+  string_vector *list = conn ? &conn->interns : &global_interns;
+
+  if (string_vector_append (list, str) == -1) {
+    nbdkit_error ("malloc: %m");
+    free (str);
+    return NULL;
+  }
+
+  return str;
+}
+
+const char *
+nbdkit_strndup_intern (const char *str, size_t n)
+{
+  char *copy;
+
+  if (str == NULL) {
+    nbdkit_error ("nbdkit_strndup_intern: no string given");
+    errno = EINVAL;
+    return NULL;
+  }
+
+  copy = strndup (str, n);
+  if (copy == NULL) {
+    nbdkit_error ("strndup: %m");
+    return NULL;
+  }
+
+  return add_intern (copy);
+}
+
+const char *
+nbdkit_strdup_intern (const char *str)
+{
+  char *copy;
+
+  if (str == NULL) {
+    nbdkit_error ("nbdkit_strdup_intern: no string given");
+    errno = EINVAL;
+    return NULL;
+  }
+
+  copy = strdup (str);
+  if (copy == NULL) {
+    nbdkit_error ("strdup: %m");
+    return NULL;
+  }
+
+  return add_intern (copy);
+}
+
+const char *
+nbdkit_vprintf_intern (const char *fmt, va_list ap)
+{
+  char *str = NULL;
+
+  if (vasprintf (&str, fmt, ap) == -1) {
+    nbdkit_error ("asprintf: %m");
+    return NULL;
+  }
+
+  return add_intern (str);
+}
+
+const char *
+nbdkit_printf_intern (const char *fmt, ...)
+{
+  va_list ap;
+  const char *ret;
+
+  va_start (ap, fmt);
+  ret = nbdkit_vprintf_intern (fmt, ap);
+  va_end (ap);
+  return ret;
+}
