@@ -38,15 +38,29 @@
 #include <inttypes.h>
 #include <string.h>
 #include <unistd.h>
-#include <poll.h>
 #include <errno.h>
 #include <assert.h>
 #include <sys/types.h>
+
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+
+#ifdef HAVE_SYS_UN_H
 #include <sys/un.h>
+#endif
+
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+
+#ifdef HAVE_NETINET_TCP_H
 #include <netinet/tcp.h>
+#endif
+
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 
 #ifdef HAVE_LINUX_VM_SOCKETS_H
 #include <linux/vm_sockets.h>
@@ -59,6 +73,7 @@
 #include <pthread.h>
 
 #include "internal.h"
+#include "poll.h"
 #include "utils.h"
 #include "vector.h"
 
@@ -102,7 +117,9 @@ bind_unix_socket (sockets *socks)
   struct sockaddr_un addr;
 
   assert (unixsocket);
+#ifndef WIN32 /* On Win32 the abspath might start with a drive letter. */
   assert (unixsocket[0] == '/');
+#endif
 
   len = strlen (unixsocket);
   if (len >= UNIX_PATH_MAX) {
@@ -207,7 +224,7 @@ bind_tcpip_socket (sockets *socks)
     if (bind (sock, a->ai_addr, a->ai_addrlen) == -1) {
       if (errno == EADDRINUSE) {
         addr_in_use = true;
-        close (sock);
+        closesocket (sock);
         continue;
       }
       perror ("bind");
@@ -402,7 +419,7 @@ accept_connection (int listen_sock)
   pthread_attr_destroy (&attrs);
   if (unlikely (err != 0)) {
     fprintf (stderr, "%s: pthread_create: %s\n", program_name, strerror (err));
-    close (thread_data->sock);
+    closesocket (thread_data->sock);
     free (thread_data);
     return;
   }
@@ -412,6 +429,8 @@ accept_connection (int listen_sock)
    */
 }
 
+#ifndef WIN32
+
 /* Check the list of sockets plus quit_fd until a POLLIN event occurs
  * on any of them.
  *
@@ -465,6 +484,52 @@ check_sockets_and_quit_fd (const sockets *socks)
   }
 }
 
+#else /* WIN32 */
+
+static void
+check_sockets_and_quit_fd (const sockets *socks)
+{
+  const size_t nr_socks = socks->size;
+  size_t i;
+  HANDLE h, handles[nr_socks+1];
+  DWORD r;
+
+  for (i = 0; i < nr_socks; ++i) {
+    h = WSACreateEvent ();
+    WSAEventSelect (_get_osfhandle (socks->ptr[i]), h,
+                    FD_ACCEPT|FD_READ|FD_CLOSE);
+    handles[i] = h;
+  }
+  handles[nr_socks] = quit_fd;
+
+  r = WaitForMultipleObjectsEx ((DWORD) (nr_socks+1), handles,
+                                FALSE, INFINITE, TRUE);
+  debug ("WaitForMultipleObjectsEx returned %d", (int) r);
+  if (r == WAIT_FAILED) {
+    fprintf (stderr, "%s: WaitForMultipleObjectsEx: error %lu\n",
+             program_name, GetLastError ());
+    exit (EXIT_FAILURE);
+  }
+
+  for (i = 0; i < nr_socks; ++i) {
+    WSAEventSelect (_get_osfhandle (socks->ptr[i]), NULL, 0);
+    WSACloseEvent (handles[i]);
+  }
+
+  if (r == WAIT_OBJECT_0 + nr_socks) /* quit_fd signalled. */
+    return;
+
+  if (r >= WAIT_OBJECT_0 && r < WAIT_OBJECT_0 + nr_socks) {
+    i = r - WAIT_OBJECT_0;
+    accept_connection (socks->ptr[i]);
+    return;
+  }
+
+  debug ("WaitForMultipleObjectsEx: unexpected return value: %lu\n", r);
+}
+
+#endif /* WIN32 */
+
 void
 accept_incoming_connections (const sockets *socks)
 {
@@ -488,6 +553,6 @@ accept_incoming_connections (const sockets *socks)
   pthread_mutex_unlock (&count_mutex);
 
   for (i = 0; i < socks->size; ++i)
-    close (socks->ptr[i]);
+    closesocket (socks->ptr[i]);
   free (socks->ptr);
 }
