@@ -1,4 +1,4 @@
-"nbdkit [-D|--debug PLUGIN|FILTER.FLAG=N]\n"
+"nbdkit [-D|--debug PLUGIN|FILTER|nbdkit.FLAG=N]\n"
 "       [-e|--exportname EXPORTNAME] [--exit-with-parent]\n"
 "       [--filter FILTER ...] [-f|--foreground]\n"
 "       [-g|--group GROUP] [-i|--ipaddr IPADDR]\n"
@@ -6,7 +6,7 @@
 "       [-n|--newstyle] [--mask-handshake MASK] [--no-sr] [-o|--oldstyle]\n"
 "       [-P|--pidfile PIDFILE]\n"
 "       [-p|--port PORT] [-r|--readonly]\n"
-"       [--run CMD] [-s|--single] [--selinux-label LABEL]\n"
+"       [--run CMD] [-s|--single] [--selinux-label LABEL] [--swap]\n"
 "       [-t|--threads THREADS]\n"
 "       [--tls off|on|require]\n"
 "       [--tls-certificates /path/to/certificates]\n"
