@@ -30,25 +30,27 @@
 # OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 # SUCH DAMAGE.
 
+# Test nbdkit --run (captive nbdkit) option.
+
 source ./functions.sh
 set -e
 set -x
 
-# Test nbdkit --run (captive nbdkit) option.
+requires_run
 
 fail=0
 
-sock=`mktemp -u`
+sock=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
 files="$sock captive.out captive.pid"
 rm -f $files
 cleanup_fn rm -f $files
 
 nbdkit -U $sock example1 --run '
-    echo nbd=$nbd; echo port=$port; echo socket=$unixsocket
+    echo uri=$uri; echo port=$port; echo socket=$unixsocket
   ' > captive.out
 
 # Check the output.
-if [ "$(cat captive.out)" != "nbd=nbd:unix:$sock
+if [ "$(cat captive.out)" != "uri=nbd+unix://?socket=$sock
 port=
 socket=$sock" ]; then
     echo "$0: unexpected output"
