@@ -48,7 +48,9 @@ if test ! -f "$script"; then
     exit 1
 fi
 
+requires_plugin cc
 requires $OCAMLOPT -version
+requires_nbdsh_uri
 requires nbdinfo --version
 
 out=test-cc-ocaml.out
