@@ -1,6 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013 Red Hat Inc.
- * All rights reserved.
+ * Copyright (C) 2013-2020 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -44,16 +43,40 @@
 
 #include "test.h"
 
+static void do_test (void);
+
 int
 main (int argc, char *argv[])
+{
+  static const char disk[] = "disk.gz";
+
+  if (access (disk, F_OK) == -1) {
+    fprintf (stderr, "%s: %s not found, test skipped\n", argv[0], disk);
+    exit (77);
+  }
+
+  /* Test the deprecated plugin - remove this when we remove the plugin. */
+  if (test_start_nbdkit ("gzip", disk, NULL) == -1)
+    exit (EXIT_FAILURE);
+  do_test ();
+
+  /* Test the new filter. */
+  if (test_start_nbdkit ("file", "--filter=gzip", disk,
+                         "fadvise=sequential", "cache=none",
+                         NULL) == -1)
+    exit (EXIT_FAILURE);
+  do_test ();
+
+  exit (EXIT_SUCCESS);
+}
+
+static void
+do_test (void)
 {
   guestfs_h *g;
   int r;
   char *data;
 
-  if (test_start_nbdkit ("gzip", "-r", "file=disk.gz", NULL) == -1)
-    exit (EXIT_FAILURE);
-
   g = guestfs_create ();
   if (g == NULL) {
     perror ("guestfs_create");
@@ -89,5 +112,4 @@ main (int argc, char *argv[])
   }
 
   guestfs_close (g);
-  exit (EXIT_SUCCESS);
 }
