@@ -40,22 +40,3 @@ if [[ ! ( "$output" =~ dump-config ) ]]; then
     echo "$output"
     exit 1
 fi
-
-# Run nbdkit plugin --help for each plugin.
-# However some of these tests are expected to fail.
-do_test ()
-{
-    vg=; [ "$NBDKIT_VALGRIND" = "1" ] && vg="-valgrind"
-    case "$1$vg" in
-        vddk | vddk-valgrind)
-            echo "$0: skipping $1$vg because VDDK cannot run without special environment variables"
-            ;;
-        python-valgrind | ruby-valgrind | tcl-valgrind)
-            echo "$0: skipping $1$vg because this language doesn't support valgrind"
-            ;;
-        *)
-            nbdkit $1 --help
-            ;;
-    esac
-}
-foreach_plugin do_test
