@@ -37,8 +37,9 @@ source ./functions.sh
 set -e
 set -x
 
+requires_run
 requires qemu-img --version
-requires nbdsh -c 'exit(not h.supports_uri())'
+requires_nbdsh_uri
 
 function do_test_info ()
 {
