@@ -38,10 +38,10 @@ requires_plugin ondemand
 requires guestfish --version
 requires qemu-img --version
 
-dir=`mktemp -d`
+dir=$(mktemp -d /tmp/nbdkit-test-dir.XXXXXX)
 cleanup_fn rm -rf $dir
 
-sock=`mktemp -u`
+sock=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
 files="ondemand.pid $sock"
 rm -f $files
 cleanup_fn rm -f $files
