@@ -40,7 +40,7 @@ set -x
 
 requires guestfish --version
 
-sock=`mktemp -u`
+sock=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
 files="partitioning3.pid $sock partitioning3.p1 partitioning3.p2 partitioning3.p3 partitioning3.out"
 rm -f $files
 cleanup_fn rm -f $files
