@@ -42,7 +42,8 @@ source ./functions.sh
 set -e
 set -x
 
-requires qemu-img --version
+requires_run
+requires nbdcopy --version
 
 # This test requires the partitioning plugin to open at least 768
 # files (say 800 to make it a round number).  On OpenBSD the limit on
@@ -77,7 +78,7 @@ nbdkit -f -v -D partitioning.regions=1 -U - \
              $d/part.* \
              partition-type=gpt \
              partition=250 \
-             --run "qemu-img convert \$nbd $d/out"
+             --run "nbdcopy \$uri $d/out"
 
 # The output should be identical to partition 250.
 cmp $d/part.0250 $d/out
