@@ -34,7 +34,8 @@ source ./functions.sh
 set -e
 set -x
 
-requires qemu-img --version
+requires_plugin sh
+requires nbdcopy --version
 requires dd iflag=count_bytes </dev/null
 
 files="retry.img retry-count retry-open-count"
@@ -48,7 +49,7 @@ start_t=$SECONDS
 nbdkit -v -U - \
        sh - \
        --filter=retry retry-delay=1 \
-       --run 'qemu-img convert $nbd retry.img' <<'EOF'
+       --run 'nbdcopy "$uri" retry.img' <<'EOF'
 #!/usr/bin/env bash
 case "$1" in
     open)
