@@ -33,13 +33,20 @@
 source ./functions.sh
 set -x
 
+# Cannot use kill pidfile below to test if the process is running on
+# Windows.
+if is_windows; then
+    echo "$0: this test needs to be revised to work on Windows"
+    exit 77
+fi
+
 requires qemu-img --version
 requires qemu-io --version
 
-plugin=.libs/test-shutdown-plugin.so
+plugin=.libs/test-shutdown-plugin.$SOEXT
 requires test -f $plugin
 
-sock=`mktemp -u`
+sock=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
 files="shutdown.pid $sock"
 cleanup_fn rm -f $files
 
