@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
 # Copyright (C) 2014 Red Hat Inc.
-# All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -31,23 +30,32 @@
 # OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 # SUCH DAMAGE.
 
+# Test nbdkit starts up, forks in the background, writes a PID file,
+# and can be killed.
+
 source ./functions.sh
 set -e
 set -x
 
-# Test nbdkit starts up, forks in the background, writes a PID file,
-# and can be killed.
+# Cannot use kill pidfile below to test if the process is running on
+# Windows.
+if is_windows; then
+    echo "$0: this test needs to be revised to work on Windows"
+    exit 77
+fi
 
-rm -f start.pid start.sock
+sock=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
+rm -f start.pid $sock
+cleanup_fn rm -f start.pid $sock
 
-start_nbdkit -P start.pid -U start.sock example1
+start_nbdkit -P start.pid -U $sock example1
 pid="$(cat start.pid)"
 
 # Check the process exists.
 kill -s 0 $pid
 
 # Check the socket was created (and is a socket).
-test -S start.sock
+test -S $sock
 
 # Kill the process.
 kill $pid
@@ -63,5 +71,3 @@ if kill -s 0 $pid; then
     echo "$0: process did not exit after sending a signal"
     exit 1
 fi
-
-rm start.pid start.sock
