@@ -33,13 +33,13 @@
 source ./functions.sh
 set -xe
 
-requires nbdsh -c 'exit(not h.supports_uri())'
+requires_nbdsh_uri
 
-plugin=.libs/test-stdio-plugin.so
+plugin=.libs/test-stdio-plugin.$SOEXT
 requires test -f $plugin
 
-sock1=`mktemp -u`
-sock2=`mktemp -u`
+sock1=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
+sock2=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
 files="test-stdio.in test-stdio.out test-stdio.err
   test-stdio.pid1 test-stdio.pid2 $sock1 $sock2"
 rm -f $files
