@@ -55,22 +55,23 @@ if [ ! -f "$pkidir/ca-cert.pem" ]; then
     exit 77
 fi
 
-# Unfortunately qemu cannot do TLS over a Unix domain socket (nbdkit
-# probably can, although it is not tested).  Find an unused port to
+# Unfortunately qemu 4.0 cannot do TLS over a Unix domain socket (nbdkit
+# can, but that is tested in tests-nbd-tls.sh).  Find an unused port to
 # listen on.
 pick_unused_port
 
 cleanup_fn rm -f tls.pid tls.out
-start_nbdkit -P tls.pid -p $port -n --tls=require \
-       --tls-certificates="$pkidir" example1
+start_nbdkit -P tls.pid -p $port -n \
+             --tls=require --tls-certificates="$pkidir" --tls-verify-peer \
+             -D nbdkit.tls.session=1 \
+             example1
 
 # Run qemu-img against the server.
-LANG=C \
-qemu-img info \
+qemu-img info --output=json \
          --object "tls-creds-x509,id=tls0,endpoint=client,dir=$pkidir" \
          --image-opts "file.driver=nbd,file.host=localhost,file.port=$port,file.tls-creds=tls0" > tls.out
 
 cat tls.out
 
-grep -sq "^file format: raw" tls.out
-grep -sq "^virtual size: 100M" tls.out
+grep -sq '"format": *"raw"' tls.out
+grep -sq '"virtual-size": *104857600\b' tls.out
