@@ -34,7 +34,8 @@ source ./functions.sh
 set -e
 set -x
 
-requires nbdsh -c 'exit(not h.supports_uri())'
+requires_plugin tmpdisk
+requires_nbdsh_uri
 
 # - If multiple parameters appear, last one is used.
 # - Test quoting.
